@@ -1,18 +1,16 @@
 # -*- coding: utf-8 -*-
 
-'''
+"""
 http://www.python.org/topics/database/DatabaseAPI-2.0.html
-'''
+"""
 
 import datetime
 import glob
 import os
 import sys
 import time
-import traceback
 import warnings
 import decimal
-import binascii
 import jpype
 from .sqloption import SQLOptions
 
@@ -48,6 +46,11 @@ _jdbc_const_to_name = []
 
 _sqloptions = SQLOptions()
 
+# 默认情况下blob字段内容输出的最大长度
+_blobdefaultfetchsize = 20
+# 默认情况下clob字段内容输出的最大长度
+_clobdefaultfetchsize = 20
+
 
 def _handle_sql_exception_jpype():
     import jpype
@@ -65,7 +68,7 @@ def _handle_sql_exception_jpype():
 
 def _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs):
     import jpype
-    if not jpype.isJVMStarted():
+    if not getattr(jpype, "isJVMStarted")():
         args = []
         class_path = []
         if jars:
@@ -78,10 +81,10 @@ def _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs):
             # path to shared libraries
             libs_path = os.path.pathsep.join(libs)
             args.append('-Djava.library.path=%s' % libs_path)
-        jvm_path = jpype.getDefaultJVMPath()
-        jpype.startJVM(jvm_path, *args, ignoreUnrecognized=True, convertStrings=True)
-    if not jpype.isThreadAttachedToJVM():
-        jpype.attachThreadToJVM()
+        jvm_path = getattr(jpype, "getDefaultJVMPath")()
+        getattr(jpype, "startJVM")(jvm_path, *args, ignoreUnrecognized=True, convertStrings=True)
+    if not getattr(jpype, "isThreadAttachedToJVM")():
+        getattr(jpype, "attachThreadToJVM")()
         jpype.java.lang.Thread.currentThread().setContextClassLoader(jpype.java.lang.ClassLoader.getSystemClassLoader())
     if _jdbc_name_to_const is None:
         types = jpype.java.sql.Types
@@ -95,7 +98,7 @@ def _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs):
     # register driver for DriverManager
     try:
         jpype.JClass(jclassname)
-    except TypeError as te:
+    except TypeError:
         raise SQLCliJDBCException("SQLCLI-00000: Load java class failed. [" + str(jclassname) + "]")
     if isinstance(driver_args, dict):
         Properties = jpype.java.util.Properties
@@ -109,10 +112,9 @@ def _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs):
     try:
         return jpype.java.sql.DriverManager.getConnection(url, *dargs)
     except jpype.java.sql.SQLException as je:
-        raise SQLCliJDBCException(je.toString().
-                              replace("java.sql.SQLException: ", "").
-                              replace("java.sql.SQLTransientConnectionException: ", "").
-                              replace("java.sql.SQLInvalidAuthorizationSpecException: ", ""))
+        raise SQLCliJDBCException(je.toString().replace("java.sql.SQLException: ", "").
+                                  replace("java.sql.SQLTransientConnectionException: ", "").
+                                  replace("java.sql.SQLInvalidAuthorizationSpecException: ", ""))
 
 
 def _get_classpath():
@@ -209,10 +211,6 @@ class Error(Exception):
     pass
 
 
-class Warning(Exception):
-    pass
-
-
 class InterfaceError(Error):
     pass
 
@@ -264,8 +262,20 @@ Time = _str_func(datetime.time)
 Timestamp = _str_func(datetime.datetime)
 
 
+def setBlobDefaultFetchSize(p_nBlobDefaultFetchSize):
+    # 程序用到的各种会话控制参数
+    global _blobdefaultfetchsize
+    _blobdefaultfetchsize = p_nBlobDefaultFetchSize
+
+
+def setClobDefaultFetchSize(p_nClobDefaultFetchSize):
+    # 程序用到的各种会话控制参数
+    global _clobdefaultfetchsize
+    _clobdefaultfetchsize = p_nClobDefaultFetchSize
+
+
 # DB-API 2.0 Module Interface connect constructor
-def connect(jclassname, url, driver_args=None, jars=None, libs=None, sqloptions=None):
+def connect(jclassname, url, driver_args=None, jars=None, libs=None):
     """Open a connection to a database using a JDBC driver and return
     a Connection instance.
 
@@ -297,29 +307,8 @@ def connect(jclassname, url, driver_args=None, jars=None, libs=None, sqloptions=
     else:
         libs = []
 
-    # 程序用到的各种会话控制参数
-    global _sqloptions
-    _sqloptions = sqloptions
-
     # 如果无法连接，则尝试3次，间隔等待2秒
-    jconn = None
-    retryCount = 0
-    while True:
-        try:
-            jconn = _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs)
-            break
-        except SQLCliJDBCException as je:
-            if jconn is None:
-                # jconn 为空，可能是网络错误，这里重复尝试
-                if "SQLCLI_DEBUG" in os.environ:
-                    print('traceback.print_exc():\n%s' % traceback.print_exc())
-                    print('traceback.format_exc():\n%s' % traceback.format_exc())
-                retryCount = retryCount + 1
-                if retryCount >= int(_sqloptions.get("CONN_RETRY_TIMES")):
-                    raise je
-                else:
-                    time.sleep(2)
-                    continue
+    jconn = _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs)
     return Connection(jconn, _converters)
 
 
@@ -353,6 +342,9 @@ class Connection(object):
         self._closed = False
         self._converters = converters
 
+    def isClosed(self):
+        return self._closed
+
     def close(self):
         if self.jconn is not None:
             self.jconn.close()
@@ -361,13 +353,13 @@ class Connection(object):
     def commit(self):
         try:
             self.jconn.commit()
-        except:
+        except Exception:
             _handle_sql_exception_jpype()
 
     def rollback(self):
         try:
             self.jconn.rollback()
-        except:
+        except Exception:
             _handle_sql_exception_jpype()
 
     def cursor(self):
@@ -379,12 +371,16 @@ class Connection(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
+    def setAutoCommit(self, p_bAutoCommit):
+        self.jconn.setAutoCommit(p_bAutoCommit)
+
 
 # DB-API 2.0 Cursor Object
 class Cursor(object):
     rowcount = -1
     _meta = None
     _prep = None
+    _stmt = None
     _rs = None
     _description = None
     warnings = None
@@ -393,6 +389,17 @@ class Cursor(object):
         self._connection = connection
         self._converters = converters
 
+    # 取消当前正在执行的SQL语句
+    def cancel(self):
+        try:
+            if self._prep is not None:
+                self._prep.cancel()
+            if self._stmt is not None:
+                self._stmt.cancel()
+        except Exception:
+            # 并不处理任何cancel时候发生的错误
+            pass
+
     @property
     def description(self):
         if self._description:
@@ -407,11 +414,14 @@ class Cursor(object):
                 if jdbc_type == 0:
                     # PEP-0249: SQL NULL values are represented by the
                     # Python None singleton
-                    dbapi_type = None
+                    column_type = None
                 else:
-                    dbapi_type = DBAPITypeObject._map_jdbc_type_to_dbapi(jdbc_type)
-                col_desc = (m.getColumnName(col),
-                            dbapi_type,
+                    if jdbc_type not in _jdbc_const_to_name:
+                        column_type = "UNKNOWN"
+                    else:
+                        column_type = str(_jdbc_const_to_name[jdbc_type])
+                col_desc = (m.getColumnLabel(col),
+                            column_type,
                             size,
                             size,
                             m.getPrecision(col),
@@ -422,7 +432,6 @@ class Cursor(object):
             return self._description
 
     #   optional callproc(self, procname, *parameters) unsupported
-
     def close(self):
         self._close_last()
         self._connection = None
@@ -440,38 +449,51 @@ class Cursor(object):
         self._description = None
 
     def _set_stmt_parms(self, prep_stmt, parameters):
+        if self:
+            pass
         for i in range(len(parameters)):
-            prep_stmt.setObject(i + 1, parameters[i])
+            if type(parameters[i]) == list:
+                m_JavaObject = None
+                if len(parameters[i]) != 0:
+                    m_JavaArray = jpype.JArray(jpype.JObject)
+                    m_JavaObject = m_JavaArray(parameters[i])
+                prep_stmt.setObject(i + 1, m_JavaObject)
+            else:
+                prep_stmt.setObject(i + 1, parameters[i])
 
     def execute_direct(self, operation):
-        if self._connection._closed:
+        if self._connection.isClosed():
             raise Error()
         self._close_last()
         is_rs = False
-        m_stmt = self._connection.jconn.createStatement()
+        self._stmt = self._connection.jconn.createStatement()
         try:
-            is_rs = m_stmt.execute(operation)
-            m_SQLWarnings = m_stmt.getWarnings()
-            if m_SQLWarnings is not None:
-                self.warnings = m_SQLWarnings.getMessage()
-            else:
-                self.warnings = None
-        except:
+            is_rs = self._stmt.execute(operation)
+            m_SQLWarnMessage = None
+            m_SQLWarnings = self._stmt.getWarnings()
+            while m_SQLWarnings is not None:
+                if m_SQLWarnMessage is None:
+                    m_SQLWarnMessage = m_SQLWarnings.getMessage()
+                else:
+                    m_SQLWarnMessage = m_SQLWarnMessage + "\n" + m_SQLWarnings.getMessage()
+                m_SQLWarnings = m_SQLWarnings.getNextWarning()
+            self.warnings = m_SQLWarnMessage
+        except Exception:
             _handle_sql_exception_jpype()
-        self._rs = m_stmt.getResultSet()
+        self._rs = self._stmt.getResultSet()
         if self._rs is not None:
             self._meta = self._rs.getMetaData()
-            self.rowcount = m_stmt.getUpdateCount()
+            self.rowcount = self._stmt.getUpdateCount()
         else:
             self._meta = None
             self.rowcount = -1
         if is_rs:
             self.rowcount = -1
         else:
-            self.rowcount = m_stmt.getUpdateCount()
+            self.rowcount = self._stmt.getUpdateCount()
 
     def execute(self, operation, parameters=None):
-        if self._connection._closed:
+        if self._connection.isClosed():
             raise Error()
         if not parameters:
             parameters = ()
@@ -481,12 +503,16 @@ class Cursor(object):
         is_rs = False
         try:
             is_rs = self._prep.execute()
+            m_SQLWarnMessage = None
             m_SQLWarnings = self._prep.getWarnings()
-            if m_SQLWarnings is not None:
-                self.warnings = m_SQLWarnings.getMessage()
-            else:
-                self.warnings = None
-        except:
+            while m_SQLWarnings is not None:
+                if m_SQLWarnMessage is None:
+                    m_SQLWarnMessage = m_SQLWarnings.getMessage()
+                else:
+                    m_SQLWarnMessage = m_SQLWarnMessage + "\n" + m_SQLWarnings.getMessage()
+                m_SQLWarnings = m_SQLWarnings.getNextWarning()
+            self.warnings = m_SQLWarnMessage
+        except Exception:
             _handle_sql_exception_jpype()
         # 忽略对execute的返回判断，总是恒定的去调用getResultSet
         self._rs = self._prep.getResultSet()
@@ -506,7 +532,15 @@ class Cursor(object):
             self._set_stmt_parms(self._prep, parameters)
             self._prep.addBatch()
         update_counts = self._prep.executeBatch()
-        # self._prep.getWarnings() ???
+        m_SQLWarnMessage = None
+        m_SQLWarnings = self._prep.getWarnings()
+        while m_SQLWarnings is not None:
+            if m_SQLWarnMessage is None:
+                m_SQLWarnMessage = m_SQLWarnings.getMessage()
+            else:
+                m_SQLWarnMessage = m_SQLWarnMessage + "\n" + m_SQLWarnings.getMessage()
+            m_SQLWarnings = m_SQLWarnings.getNextWarning()
+        self.warnings = m_SQLWarnMessage
         self.rowcount = sum(update_counts)
         self._close_last()
 
@@ -523,7 +557,7 @@ class Cursor(object):
             if m_ColumnClassName is None:
                 # NULL值
                 converter = _DEFAULT_CONVERTERS["VARCHAR"]
-            elif m_ColumnClassName in ('org.postgresql.util.PGmoney'):
+            elif m_ColumnClassName in 'org.postgresql.util.PGmoney':
                 converter = _DEFAULT_CONVERTERS["VARCHAR"]
             elif m_ColumnClassName in ('oracle.sql.TIMESTAMPTZ', 'oracle.sql.TIMESTAMPLTZ'):
                 converter = _DEFAULT_CONVERTERS["TIMESTAMP_WITH_TIMEZONE"]
@@ -593,6 +627,8 @@ class Cursor(object):
 
 
 def _unknownSqlTypeConverter(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
@@ -600,18 +636,23 @@ def _unknownSqlTypeConverter(conn, rs, col):
 
 
 def _to_datetime(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getTimestamp(col)
     if not java_val:
         return
     try:
         d = datetime.datetime.strptime(str(java_val)[:19], "%Y-%m-%d %H:%M:%S")
         d = d.replace(microsecond=int(str(java_val.getNanos())[:6]))
-    except ValueError as ve:
+    except ValueError:
+        # 处理一些超过Python数据类型限制的时间类型
         d = str(java_val)
     return str(d)
 
 
 def _to_time(conn, rs, col):
+    if conn:
+        pass
     try:
         java_val = rs.getTime(col)
     except Exception:
@@ -623,6 +664,8 @@ def _to_time(conn, rs, col):
 
 
 def _to_year(conn, rs, col):
+    if conn:
+        pass
     try:
         java_val = rs.getDate(col)
     except Exception:
@@ -636,6 +679,8 @@ def _to_year(conn, rs, col):
 
 
 def _to_date(conn, rs, col):
+    if conn:
+        pass
     try:
         java_val = rs.getDate(col)
     except Exception:
@@ -650,7 +695,10 @@ def _to_date(conn, rs, col):
     # https://github.com/baztian/jaydebeapi/issues/18):
     return str(java_val)[:10]
 
+
 def _to_bit(conn, rs, col):
+    if conn:
+        pass
     try:
         java_val = rs.getObject(col)
     except Exception:
@@ -671,10 +719,14 @@ def _to_bit(conn, rs, col):
         if str_val is None:
             return
         else:
-            if java_val == 0:
-                return "False"
+            if len(str_val) > 1:
+                # 这不是一个标准的Boolean，按照字节内容返回
+                return "0b" + str(str_val)
             else:
-                return "True"
+                if java_val == 0:
+                    return "False"
+                else:
+                    return "True"
     elif m_TypeName.find("BinaryData") != -1:
         m_Byte = java_val.getBytes()
         return "0b" + str(bin(int.from_bytes(m_Byte, sys.byteorder))).lstrip("0b").zfill(len(m_Byte)*8)
@@ -682,19 +734,64 @@ def _to_bit(conn, rs, col):
         raise SQLCliJDBCException("SQLCLI-00000: Unknown java class type [" + m_TypeName + "] in _to_bit")
 
 
-def _to_binary(conn, rs, col):
+def _to_varbinary(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
     m_TypeName = str(java_val.getClass().getTypeName())
     if m_TypeName == "byte[]":
-        return binascii.b2a_hex(java_val)
-    elif m_TypeName.upper().find("BLOB") != -1:
-        m_Bytes = java_val.getBytes(1, 1024)
-        if m_Bytes is not None:
-            return binascii.b2a_hex(java_val.getBytes(1, 1024))
+        # 截断所有后面为0的字节数组内容
+        m_TrimPos = -1
+        for m_nPos in range(len(java_val) - 1, 0, -1):
+            if java_val[m_nPos] != 0:
+                m_TrimPos = m_nPos
+                break
+        if m_TrimPos == -1:
+            return java_val
         else:
-            return ""
+            return java_val[:m_TrimPos + 1]
+    elif m_TypeName.upper().find("BLOB") != -1:
+        m_Length = java_val.length()
+        # 总是返回比LOB_LENGTH大1的字节数，后续的显示中将根据LOB_LENGTH是否超长做出是否打印省略号的标志
+        m_TrimLength = _blobdefaultfetchsize + 1
+        if m_TrimLength > m_Length:
+            m_TrimLength = m_Length
+        m_Bytes = java_val.getBytes(1, int(m_TrimLength))
+        return m_Bytes
+    elif m_TypeName.find("BFILE") != -1:
+        return "bfilename(" + java_val.getDirAlias() + ":" + java_val.getName() + ")"
+    elif m_TypeName.find("Boolean") != -1:
+        str_val = rs.getString(col)
+        if str_val is None:
+            return
+        else:
+            if java_val == 0:
+                return "False"
+            else:
+                return "True"
+    else:
+        raise SQLCliJDBCException("SQLCLI-00000: Unknown java class type [" + m_TypeName + "] in _to_binary")
+
+
+def _to_binary(conn, rs, col):
+    if conn:
+        pass
+    java_val = rs.getObject(col)
+    if java_val is None:
+        return
+    m_TypeName = str(java_val.getClass().getTypeName())
+    if m_TypeName == "byte[]":
+        return java_val
+    elif m_TypeName.upper().find("BLOB") != -1:
+        m_Length = java_val.length()
+        # 总是返回比LOB_LENGTH大1的字节数，后续的显示中将根据LOB_LENGTH是否超长做出是否打印省略号的标志
+        m_TrimLength = _blobdefaultfetchsize + 1
+        if m_TrimLength > m_Length:
+            m_TrimLength = m_Length
+        m_Bytes = java_val.getBytes(1, int(m_TrimLength))
+        return m_Bytes
     elif m_TypeName.find("BFILE") != -1:
         return "bfilename(" + java_val.getDirAlias() + ":" + java_val.getName() + ")"
     elif m_TypeName.find("Boolean") != -1:
@@ -707,12 +804,13 @@ def _to_binary(conn, rs, col):
             else:
                 return "True"
     else:
-        # return binascii.b2a_hex(java_val.getBytes(0, 1024))
         raise SQLCliJDBCException("SQLCLI-00000: Unknown java class type [" + m_TypeName + "] in _to_binary")
 
 
 def _java_to_py(java_method):
     def to_py(conn, rs, col):
+        if conn:
+            pass
         java_val = rs.getObject(col)
         if java_val is None:
             return
@@ -723,6 +821,8 @@ def _java_to_py(java_method):
 
 
 def _java_to_py_bigdecimal(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
@@ -753,7 +853,7 @@ def _java_to_py_timestampwithtimezone(conn, rs, col):
     elif m_TypeName in ('oracle.sql.TIMESTAMPTZ', 'oracle.sql.TIMESTAMPLTZ'):
         return java_val.offsetDateTimeValue(conn).format(
             jpype.java.time.format.DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSSSSS Z"))
-    elif m_TypeName in ('org.h2.api.TimestampWithTimeZone'):
+    elif m_TypeName in 'org.h2.api.TimestampWithTimeZone':
         return java_val.toString()
     else:
         raise SQLCliJDBCException(
@@ -762,13 +862,15 @@ def _java_to_py_timestampwithtimezone(conn, rs, col):
 
 
 def _java_to_py_clob(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
     m_TypeName = str(java_val.getClass().getTypeName())
     if m_TypeName.upper().find('CLOB') != -1:
         m_Length = java_val.length()
-        m_TrimLength = int(_sqloptions.get("LOB_LENGTH"))
+        m_TrimLength = _clobdefaultfetchsize
         if m_TrimLength > m_Length:
             m_TrimLength = m_Length
         m_ColumnValue = java_val.getSubString(1, int(m_TrimLength))
@@ -782,54 +884,15 @@ def _java_to_py_clob(conn, rs, col):
 
 
 def _java_to_py_stru(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
     m_TypeName = str(java_val.getClass().getTypeName())
-    if (m_TypeName == "java.lang.Object[]"):
-        m_ColumnValue = "STRUCTURE("
-        for m_nPos in range(0, len(java_val)):
-            m_ColumnType = str(type(java_val[m_nPos]))
-            if m_nPos == 0:
-                if m_ColumnType.upper().find('STR') != -1:
-                    m_ColumnValue = m_ColumnValue + "'" + str(java_val[m_nPos]) + "'"
-                elif m_ColumnType.upper().find('SQLDATE') != -1:
-                    m_ColumnValue = m_ColumnValue + "DATE'" + str(java_val[m_nPos]) + "'"
-                elif str(type(java_val)).upper().find("FLOAT") != -1:
-                    m_ColumnValue = m_ColumnValue + \
-                                    _sqloptions.get("FLOAT_FORMAT") % java_val[m_nPos]
-                elif str(type(java_val)).upper().find("DOUBLE") != -1:
-                    m_ColumnValue = m_ColumnValue + \
-                                    _sqloptions.get("DOUBLE_FORMAT") % java_val[m_nPos]
-                elif type(java_val) == decimal.Decimal:
-                    if _sqloptions.get("DECIMAL_FORMAT") != "":
-                        m_ColumnValue = m_ColumnValue + \
-                                        _sqloptions.get("DECIMAL_FORMAT") % java_val[m_nPos]
-                    else:
-                        m_ColumnValue = m_ColumnValue + java_val[m_nPos]
-                else:
-                    m_ColumnValue = m_ColumnValue + str(java_val[m_nPos])
-            else:
-                if m_ColumnType.upper().find('STR') != -1:
-                    m_ColumnValue = m_ColumnValue + ",'" + str(java_val[m_nPos]) + "'"
-                elif m_ColumnType.upper().find('SQLDATE') != -1:
-                    m_ColumnValue = m_ColumnValue + ",DATE'" + str(java_val[m_nPos]) + "'"
-                elif str(type(java_val)).upper().find("FLOAT") != -1:
-                    m_ColumnValue = m_ColumnValue + "," + \
-                                    _sqloptions.get("FLOAT_FORMAT") % java_val[m_nPos]
-                elif str(type(java_val)).upper().find("DOUBLE") != -1:
-                    m_ColumnValue = m_ColumnValue + "," + \
-                                    _sqloptions.get("DOUBLE_FORMAT") % java_val[m_nPos]
-                elif type(java_val) == decimal.Decimal:
-                    if _sqloptions.get("DECIMAL_FORMAT") != "":
-                        m_ColumnValue = m_ColumnValue + "," + \
-                                        _sqloptions.get("DECIMAL_FORMAT") % java_val[m_nPos]
-                    else:
-                        m_ColumnValue = m_ColumnValue + "," + java_val[m_nPos]
-                else:
-                    m_ColumnValue = m_ColumnValue + "," + str(java_val[m_nPos])
-        m_ColumnValue = m_ColumnValue + ")"
-        return m_ColumnValue
+    if m_TypeName == "java.lang.Object[]":
+        m_retVal = tuple(java_val)
+        return m_retVal
     else:
         raise SQLCliJDBCException(
             "SQLCLI-00000: Unknown java class type [" + m_TypeName +
@@ -837,55 +900,21 @@ def _java_to_py_stru(conn, rs, col):
 
 
 def _java_to_py_array(conn, rs, col):
+    if conn:
+        pass
     java_val = rs.getObject(col)
     if java_val is None:
         return
     m_TypeName = str(java_val.getClass().getTypeName())
     if m_TypeName.upper().find('ARRAY') != -1:
-        m_ColumnValue = "ARRAY["
-        java_val = java_val.getArray();
-        for m_nPos in range(0, len(java_val)):
-            m_ColumnType = str(type(java_val[m_nPos]))
-            if m_nPos == 0:
-                if m_ColumnType.upper().find('STR') != -1:
-                    m_ColumnValue = m_ColumnValue + "'" + str(java_val[m_nPos]) + "'"
-                elif m_ColumnType.upper().find('SQLDATE') != -1:
-                    m_ColumnValue = m_ColumnValue + "DATE'" + str(java_val[m_nPos]) + "'"
-                elif str(type(java_val)).upper().find("FLOAT") != -1:
-                    m_ColumnValue = m_ColumnValue + \
-                                    _sqloptions.get("FLOAT_FORMAT") % java_val[m_nPos]
-                elif str(type(java_val)).upper().find("DOUBLE") != -1:
-                    m_ColumnValue = m_ColumnValue + \
-                                    _sqloptions.get("DOUBLE_FORMAT") % java_val[m_nPos]
-                elif type(java_val) == decimal.Decimal:
-                    if _sqloptions.get("DECIMAL_FORMAT") != "":
-                        m_ColumnValue = m_ColumnValue + \
-                                        _sqloptions.get("DECIMAL_FORMAT") % java_val[m_nPos]
-                    else:
-                        m_ColumnValue = m_ColumnValue + java_val[m_nPos]
-                else:
-                    m_ColumnValue = m_ColumnValue + str(java_val[m_nPos])
-            else:
-                if m_ColumnType.upper().find('STR') != -1:
-                    m_ColumnValue = m_ColumnValue + ",'" + str(java_val[m_nPos]) + "'"
-                elif m_ColumnType.upper().find('SQLDATE') != -1:
-                    m_ColumnValue = m_ColumnValue + ",DATE'" + str(java_val[m_nPos]) + "'"
-                elif str(type(java_val)).upper().find("FLOAT") != -1:
-                    m_ColumnValue = m_ColumnValue + "," + \
-                                    _sqloptions.get("FLOAT_FORMAT") % java_val[m_nPos]
-                elif str(type(java_val)).upper().find("DOUBLE") != -1:
-                    m_ColumnValue = m_ColumnValue + "," + \
-                                    _sqloptions.get("DOUBLE_FORMAT") % java_val[m_nPos]
-                elif type(java_val) == decimal.Decimal:
-                    if _sqloptions.get("DECIMAL_FORMAT") != "":
-                        m_ColumnValue = m_ColumnValue + "," + \
-                                        _sqloptions.get("DECIMAL_FORMAT") % java_val[m_nPos]
-                    else:
-                        m_ColumnValue = m_ColumnValue + "," + java_val[m_nPos]
-                else:
-                    m_ColumnValue = m_ColumnValue + "," + str(java_val[m_nPos])
-        m_ColumnValue = m_ColumnValue + "]"
-        return m_ColumnValue
+        java_val = java_val.getArray()
+        m_retVal = []
+        m_retVal.extend(java_val)
+        return m_retVal
+    elif m_TypeName.find('Object[]') != -1:
+        m_retVal = []
+        m_retVal.extend(java_val)
+        return m_retVal
     else:
         raise SQLCliJDBCException(
             "SQLCLI-00000: Unknown java class type [" + m_TypeName +
@@ -893,6 +922,8 @@ def _java_to_py_array(conn, rs, col):
 
 
 def _java_to_py_str(conn, rs, col):
+    if conn:
+        pass
     try:
         java_val = rs.getObject(col)
         if java_val is None:
@@ -939,7 +970,7 @@ _DEFAULT_CONVERTERS = {
     'TIME':                         _to_time,
     'DATE':                         _to_date,
     'YEAR':                         _to_year,
-    'VARBINARY':                    _to_binary,
+    'VARBINARY':                    _to_varbinary,
     'BINARY':                       _to_binary,
     'LONGVARBINARY':                _to_binary,
     'BLOB':                         _to_binary,
