@@ -7,6 +7,7 @@ import os
 import re
 import traceback
 import json
+import binascii
 from func_timeout import func_set_timeout
 import func_timeout
 
@@ -47,7 +48,7 @@ class SQLExecute(object):
         self.cur = None
 
         # 脚本启动的时间
-        self.StartTime = None
+        self.StartTime = time.time()
 
     def setStartTime(self, p_StartTime):
         self.StartTime = p_StartTime
@@ -264,8 +265,8 @@ class SQLExecute(object):
 
             # 记录命令开始时间
             start = time.time()
-            m_SQLTimeOut = int(self.SQLOptions.get("SQLCLI_SQL_TIMEOUT"))
-            m_ScriptTimeOut = int(self.SQLOptions.get("SQLCLI_SCRIPT_TIMEOUT"))
+            m_SQLTimeOut = int(self.SQLOptions.get("SQL_TIMEOUT"))
+            m_ScriptTimeOut = int(self.SQLOptions.get("SCRIPT_TIMEOUT"))
 
             # 执行SQL
             try:
@@ -289,20 +290,69 @@ class SQLExecute(object):
                     if "type" not in m_Result.keys():
                         m_Result.update({"type": "result"})
                     if m_Result["type"] == "result" and sql.startswith("__internal__"):
-                        # 对于internal的语句要记录影响的行列信息，同时控制TERMOUT
+                        result = m_Result["rows"]
+                        # 如果Hints中有order字样，对结果进行排序后再输出
+                        if "Order" in m_SQLHint.keys() and result is not None:
+                            for i in range(1, len(result)):
+                                for j in range(0, len(result) - i):
+                                    if str(result[j]) > str(result[j + 1]):
+                                        result[j], result[j + 1] = result[j + 1], result[j]
+
+                        # 如果Hint中存在LogFilter，则结果集中过滤指定的输出信息
+                        if "LogFilter" in m_SQLHint.keys() and result is not None:
+                            for m_SQLFilter in m_SQLHint["LogFilter"]:
+                                for item in result[:]:
+                                    if "SQLCLI_DEBUG" in os.environ:
+                                        print("Apply Filter: " + str(''.join(str(item))) + " with " + m_SQLFilter)
+                                    if re.match(m_SQLFilter, ''.join(str(item)), re.IGNORECASE):
+                                        result.remove(item)
+                                        continue
+
+                        # 如果Hint中存在LogMask,则掩码指定的输出信息
+                        if "LogMask" in m_SQLHint.keys() and result is not None:
+                            for i in range(0, len(result)):
+                                m_Output = None
+                                for j in range(0, len(result[i])):
+                                    if m_Output is None:
+                                        m_Output = str(result[i][j])
+                                    else:
+                                        m_Output = m_Output + "|||" + str(result[i][j])
+                                for m_SQLMaskString in m_SQLHint["LogMask"]:
+                                    m_SQLMask = m_SQLMaskString.split("=>")
+                                    if len(m_SQLMask) == 2:
+                                        m_SQLMaskPattern = m_SQLMask[0]
+                                        m_SQLMaskTarget = m_SQLMask[1]
+                                        if "SQLCLI_DEBUG" in os.environ:
+                                            print("Apply Mask: " + m_Output +
+                                                  " with " + m_SQLMaskPattern + "=>" + m_SQLMaskTarget)
+                                        try:
+                                            m_NewOutput = re.sub(m_SQLMaskPattern, m_SQLMaskTarget, m_Output,
+                                                                 re.IGNORECASE)
+                                            if m_NewOutput != m_Output:
+                                                result[i] = tuple(m_NewOutput.split('|||'))
+                                                m_Output = m_NewOutput
+                                        except re.error:
+                                            if "SQLCLI_DEBUG" in os.environ:
+                                                print('traceback.print_exc():\n%s' % traceback.print_exc())
+                                                print('traceback.format_exc():\n%s' % traceback.format_exc())
+                                    else:
+                                        if "SQLCLI_DEBUG" in os.environ:
+                                            print("LogMask Hint Error: " + m_SQLHint["LogMask"])
+                        m_Result["rows"] = result
                         if m_Result["rows"] is None:
                             m_Rows = 0
                         else:
                             m_Rows = len(m_Result["rows"])
-                        if self.SQLOptions.get('TERMOUT').upper() == 'OFF':
-                            m_Result["rows"] = []
-                            m_Result["title"] = ""
                         self.LastJsonSQLResult = {"desc": m_Result["headers"],
                                                   "rows": m_Rows,
                                                   "elapsed": time.time() - start,
                                                   "result": m_Result["rows"],
                                                   "status": 0,
                                                   "warnings": ""}
+                        # 如果TERMOUT为关闭，不在返回结果信息
+                        if self.SQLOptions.get('TERMOUT').upper() == 'OFF':
+                            m_Result["rows"] = []
+                            m_Result["title"] = ""
                     yield m_Result
             except CommandNotFound:
                 # 进入到SQL执行阶段, 开始执行SQL语句
@@ -376,14 +426,6 @@ class SQLExecute(object):
                                         if str(result[j]) > str(result[j + 1]):
                                             result[j], result[j + 1] = result[j + 1], result[j]
 
-                            # 保存之前的运行结果
-                            self.LastJsonSQLResult = {"desc": headers,
-                                                      "rows": rowcount,
-                                                      "elapsed": time.time() - start,
-                                                      "result": result,
-                                                      "status": 0,
-                                                      "warnings": m_SQLWarnings}
-
                             # 如果Hint中存在LogFilter，则结果集中过滤指定的输出信息
                             if "LogFilter" in m_SQLHint.keys() and result is not None:
                                 for m_SQLFilter in m_SQLHint["LogFilter"]:
@@ -425,6 +467,18 @@ class SQLExecute(object):
                                             if "SQLCLI_DEBUG" in os.environ:
                                                 print("LogMask Hint Error: " + m_SQLHint["LogMask"])
 
+                            # 保存之前的运行结果
+                            if result is None:
+                                m_Rows = 0
+                            else:
+                                m_Rows = len(result)
+                            self.LastJsonSQLResult = {"desc": headers,
+                                                      "rows": m_Rows,
+                                                      "elapsed": time.time() - start,
+                                                      "result": result,
+                                                      "status": 0,
+                                                      "warnings": m_SQLWarnings}
+
                             # 如果存在SQL_LOOP信息，则需要反复执行上一个SQL
                             if "SQL_LOOP" in m_SQLHint.keys():
                                 if "SQLCLI_DEBUG" in os.environ:
@@ -658,6 +712,7 @@ class SQLExecute(object):
         columntypes = []
         if cursor.description is not None:
             headers = [x[0] for x in cursor.description]
+            columntypes = [x[1] for x in cursor.description]
             if cursor.warnings is not None:
                 status = "{0} row{1} selected with warnings."
             else:
@@ -666,43 +721,74 @@ class SQLExecute(object):
             rowset = cursor.fetchmany(m_arraysize)
             for row in rowset:
                 m_row = []
-                # 记录字段类型
-                if len(columntypes) == 0:
-                    for column in row:
-                        if "SQLCLI_DEBUG" in os.environ:
-                            print("DEBUG: COLUMN TYPE TYPE: [" + str(type(column)) + "]")
-                        if type(column) == int:
-                            m_ColumnType = "int"
-                        elif type(column) == str:
-                            m_ColumnType = "str"
-                        elif type(column) == bool:
-                            m_ColumnType = "bool"
-                        elif type(column) == float:
-                            m_ColumnType = "float"
-                        else:
-                            m_ColumnType = str(type(column))
-                        columntypes.append(m_ColumnType)
-                        if "SQLCLI_DEBUG" in os.environ:
-                            print("DEBUG: COLUMN TYPE STR : [" + m_ColumnType + "]")
-                for column in row:
-                    if str(type(column)).upper().find('JDBCBLOBCLIENT') != -1:
-                        # 对于二进制数据，用16进制数来显示
-                        # 2: 意思是去掉Hex前面的0x字样
-                        m_Length = column.length()
-                        m_ColumnValue = "".join([hex(int(i))[2:]
-                                                 for i in column.getBytes(1,
-                                                                          int(self.SQLOptions.get("LOB_LENGTH")))])
-                        if m_Length > int(self.SQLOptions.get("LOB_LENGTH")):
-                            m_ColumnValue = m_ColumnValue + "..."
-                        m_row.append(m_ColumnValue)
-                    elif str(type(column)).upper().find("FLOAT") != -1:
+                for m_nColumnPos in range(0, len(row)):
+                    column = row[m_nColumnPos]
+                    columntype = columntypes[m_nColumnPos]
+                    # 对于空值直接返回
+                    if column is None:
+                        m_row.append(None)
+                        continue
+                    # 处理各种数据类型
+                    if columntypes[m_nColumnPos] == "FLOAT":
                         m_row.append(self.SQLOptions.get("FLOAT_FORMAT") % column)
-                    elif type(column) == decimal.Decimal:
+                    elif columntypes[m_nColumnPos] in ("DECIMAL", "DOUBLE"):
                         if self.SQLOptions.get("DECIMAL_FORMAT") != "":
                             m_row.append(self.SQLOptions.get("DECIMAL_FORMAT") % column)
                         else:
                             m_row.append(column)
-                    elif type(column) == list:
+                    elif columntypes[m_nColumnPos] == "STRUCT":
+                        m_ColumnValue = "STRUCTURE("
+                        for m_nPos in range(0, len(column)):
+                            m_ColumnType = str(type(column[m_nPos]))
+                            if m_nPos == 0:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + "'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + "DATE'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DOUBLE") != -1:
+                                    m_CellValue = decimal.Decimal(column[m_nPos].toString())
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + \
+                                                        str(self.SQLOptions.get("DECIMAL_FORMAT") % m_CellValue)
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + str(m_CellValue)
+                                elif m_ColumnType.upper().find("DECIMAL") != -1:
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + \
+                                                        str(self.SQLOptions.get("DECIMAL_FORMAT") % column[m_nPos])
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + str(column[m_nPos])
+                                else:
+                                    m_ColumnValue = m_ColumnValue + str(column[m_nPos])
+                            else:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",DATE'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + "," + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DOUBLE") != -1:
+                                    m_CellValue = decimal.Decimal(column[m_nPos].toString())
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + "," + \
+                                                        str(self.SQLOptions.get("DECIMAL_FORMAT") % m_CellValue)
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + "," + str(m_CellValue)
+                                elif m_ColumnType.upper().find("DECIMAL") != -1:
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + "," + \
+                                                        str(self.SQLOptions.get("DECIMAL_FORMAT") % column[m_nPos])
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
+                                else:
+                                    m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
+                        m_ColumnValue = m_ColumnValue + ")"
+                        m_row.append(m_ColumnValue)
+                    elif columntypes[m_nColumnPos] == "ARRAY":
                         m_ColumnValue = "ARRAY["
                         for m_nPos in range(0, len(column)):
                             m_ColumnType = str(type(column[m_nPos]))
@@ -754,20 +840,24 @@ class SQLExecute(object):
                                     m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
                         m_ColumnValue = m_ColumnValue + "]"
                         m_row.append(m_ColumnValue)
-                    elif type(column) == bytes:
-                        # 对于二进制数据，其末尾用0x00表示，这里进行截断
+                    elif columntype in ("BINARY", "VARBINARY", "LONGVARBINARY"):
+                        # 转换为16进制，并反算成ASCII
+                        column = binascii.b2a_hex(column)
                         column = column.decode()
-                        m_TrimPos = 0
-                        for m_nPos in range(len(column), 0, -2):
-                            if column[m_nPos - 2] != '0' or column[m_nPos - 1] != '0':
-                                m_TrimPos = m_nPos
-                                break
-                        if m_TrimPos != -1:
-                            column = column[0:m_TrimPos]
-                        if len(column) != 0:
-                            m_row.append("0x" + column)
+                        m_row.append("0x" + column)
+                    elif columntype == "BLOB":
+                        m_TrimLength = int(self.SQLOptions.get("LOB_LENGTH"))
+                        m_isCompleteOutput = True
+                        if len(column) > m_TrimLength:
+                            m_isCompleteOutput = False
+                            column = column[:m_TrimLength]
+                        # 转换为16进制，并反算成ASCII
+                        column = binascii.b2a_hex(column)
+                        column = column.decode()
+                        if not m_isCompleteOutput:
+                            m_row.append("0x" + column + "...")
                         else:
-                            m_row.append("")
+                            m_row.append("0x" + column)
                     else:
                         m_row.append(column)
                 m_row = tuple(m_row)
